import { SyntaxNode, Tree } from "web-tree-sitter";
import { SyntaxNodeMap } from "./utils/syntaxNodeMap";
import { IExposing, SymbolMap } from "./binder";
import { Diagnostic } from "./diagnostics";
import { ElmProject } from "./program";
export interface ISourceFile {
    uri: string;
    writeable: boolean;
    referenced: boolean;
    maintainerAndPackageName?: string;
    tree: Tree;
    project: ElmProject;
    isTestFile: boolean;
    isDependency: boolean;
    parseDiagnostics: Diagnostic[];
    bindDiagnostics: Diagnostic[];
    moduleName?: string;
    resolvedModules?: Map<string, string>;
    exposing?: IExposing;
    symbolLinks?: SyntaxNodeMap<SyntaxNode, SymbolMap>;
    nonShadowableNames?: Set<string>;
}
export interface IForest {
    treeMap: Map<string, ISourceFile>;
    getTree(uri: string): Tree | undefined;
    getByUri(uri: string): ISourceFile | undefined;
    setTree(uri: string, writeable: boolean, referenced: boolean, tree: Tree, isTestFile: boolean, isDependency: boolean, project?: ElmProject, packageName?: string): ISourceFile;
    removeTree(uri: string): void;
    synchronize(): void;
    invalidateResolvedModules(): void;
}
export declare class Forest implements IForest {
    private rootProject;
    treeMap: Map<string, ISourceFile>;
    constructor(rootProject: ElmProject);
    getTree(uri: string): Tree | undefined;
    getByUri(uri: string): ISourceFile | undefined;
    setTree(uri: string, writeable: boolean, referenced: boolean, tree: Tree, isTestFile: boolean, isDependency: boolean, project?: ElmProject, maintainerAndPackageName?: string): ISourceFile;
    removeTree(uri: string): void;
    synchronize(): void;
    invalidateResolvedModules(): void;
    private resolveModules;
    private getModuleMap;
}
