import { ExecaSyncReturnValue } from "execa";
import { Connection, CompletionItemKind } from "vscode-languageserver";
import { URI } from "vscode-uri";
import { IElmPackageCache } from "../elmPackageCache";
import { IClientSettings } from "../../util/settings";
import { ElmProject } from "../program";
import { NonEmptyArray } from "../../util/utils";
export declare const isWindows: boolean;
/** Options for execCmdSync */
export interface IExecCmdSyncOptions {
    /** Any arguments */
    cmdArguments?: string[];
    /** Text to add when command is not found (maybe helping how to install) */
    notFoundText?: string;
}
/** Executes a command. Shows an error message if the command isn't found */
export declare function execCmdSync(cmdFromUser: string, cmdStatic: string, options: IExecCmdSyncOptions | undefined, cwd: string, connection: Connection, input?: string): ExecaSyncReturnValue<string>;
/** Options for execCmd */
export interface IExecCmdOptions {
    /** Text to add when command is not found (maybe helping how to install)
     * Unlike the sync version, it’s required here (since `cmdStatic` has fallbacks).
     */
    notFoundText: string;
}
export declare function execCmd(cmdFromUser: [string, string[]], cmdStatic: NonEmptyArray<[string, string[]]>, options: IExecCmdOptions, cwd: string, connection: Connection, input?: string): Promise<ExecaSyncReturnValue<string>>;
export declare function getEmptyTypes(): {
    markdown: string;
    name: string;
    symbolKind: CompletionItemKind;
}[];
export declare function getElmVersion(settings: IClientSettings, elmWorkspaceFolder: URI, connection: Connection): string;
export declare function solveDependencies(packageCache: IElmPackageCache, deps: ReadonlyMap<string, IConstraint>): Promise<ReadonlyMap<string, IVersion> | undefined>;
export interface IVersion {
    major: number;
    minor: number;
    patch: number;
    string: string;
}
export interface IConstraint {
    upper: IVersion;
    lower: IVersion;
    upperOperator: "<" | "<=";
    lowerOperator: "<" | "<=";
}
export declare function parseVersion(version: string): IVersion;
export declare function parseConstraint(contraint: string): IConstraint;
export declare function versionSatisfiesConstraint(version: IVersion, constraint: IConstraint): boolean;
export declare function constraintIntersect(a: IConstraint, b: IConstraint): IConstraint | undefined;
export declare function getModuleName(uri: string, sourceDir: string): string;
export declare function flattenExposedModules(exposedModules: string[] | {
    [name: string]: string[];
}): string[];
export declare function nameIsKernel(name: string): boolean;
export declare function isKernelProject(project: ElmProject): boolean;
export declare function isCoreProject(project: ElmProject): boolean;
