"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElmReviewDiagnostics = exports.hasElmReviewFixes = void 0;
/* eslint-disable @typescript-eslint/no-unsafe-call */
const tsyringe_1 = require("tsyringe");
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const utils = __importStar(require("../../compiler/utils/elmUtils"));
const elmWorkspaceMatcher_1 = require("../../util/elmWorkspaceMatcher");
const fs_1 = require("fs");
const path = __importStar(require("path"));
function hasElmReviewFixes(diagnostic) {
    var _a, _b, _c, _d;
    return (((_b = (_a = diagnostic) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) === "elm_review" &&
        ((_d = (_c = diagnostic) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.fixes.length) > 0);
}
exports.hasElmReviewFixes = hasElmReviewFixes;
function toLsRange({ start, end }) {
    return {
        start: {
            character: start.column - 1,
            line: start.line - 1,
        },
        end: {
            character: end.column - 1,
            line: end.line - 1,
        },
    };
}
class ElmReviewDiagnostics {
    constructor() {
        this.createDiagnostics = (sourceFile) => __awaiter(this, void 0, void 0, function* () {
            const filePath = vscode_uri_1.URI.parse(sourceFile.uri);
            const workspaceRootPath = this.elmWorkspaceMatcher
                .getProgramFor(filePath)
                .getRootPath();
            return yield this.checkForErrors(workspaceRootPath.fsPath);
        });
        this.settings = tsyringe_1.container.resolve("Settings");
        this.connection = tsyringe_1.container.resolve("Connection");
        this.elmWorkspaceMatcher = new elmWorkspaceMatcher_1.ElmWorkspaceMatcher((uri) => uri);
    }
    hasType(error) {
        return "type" in error;
    }
    checkForErrors(workspaceRootPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.settings.getClientSettings();
            const fileErrors = new Map();
            if (settings.elmReviewDiagnostics === "off" ||
                !(0, fs_1.existsSync)(path.join(workspaceRootPath, "review", "src", "ReviewConfig.elm"))) {
                return fileErrors;
            }
            const elmReviewCommand = settings.elmReviewPath;
            const cmdArguments = ["--report", "json", "--namespace", "vscode"];
            if (settings.elmPath.trim().length > 0) {
                cmdArguments.push("--compiler", settings.elmPath);
            }
            if (settings.elmFormatPath.trim().length > 0) {
                cmdArguments.push("--elm-format-path", settings.elmFormatPath);
            }
            const options = {
                cmdArguments: cmdArguments,
                notFoundText: "'elm-review' is not available. Install elm-review via 'npm install -g elm-review'.",
            };
            try {
                // Do nothing on success, but return that there were no errors
                utils.execCmdSync(elmReviewCommand, "elm-review", options, workspaceRootPath, this.connection);
                return fileErrors;
            }
            catch (error) {
                if (typeof error === "string") {
                    return fileErrors;
                }
                else {
                    const execaError = error;
                    let errorObject;
                    try {
                        errorObject = JSON.parse(execaError.stdout);
                    }
                    catch (error) {
                        this.connection.console.warn("Received an invalid json, skipping error.");
                    }
                    if (errorObject &&
                        this.hasType(errorObject) &&
                        errorObject.type === "review-errors") {
                        errorObject.errors.forEach(({ path, errors }) => {
                            const uri = vscode_uri_1.Utils.joinPath(vscode_uri_1.URI.parse(workspaceRootPath), path).toString();
                            fileErrors.set(uri, errors
                                .filter((error) => !error.suppressed)
                                .map((error) => ({
                                message: error.message,
                                source: "elm-review",
                                range: toLsRange(error.region),
                                severity: settings.elmReviewDiagnostics === "error"
                                    ? vscode_languageserver_1.DiagnosticSeverity.Error
                                    : vscode_languageserver_1.DiagnosticSeverity.Warning,
                                tags: error.rule.startsWith("NoUnused")
                                    ? [vscode_languageserver_1.DiagnosticTag.Unnecessary]
                                    : undefined,
                                data: {
                                    uri,
                                    code: "elm_review",
                                    fixes: (error.fix || []).map((fix) => ({
                                        string: fix.string,
                                        range: toLsRange(fix.range),
                                    })),
                                },
                            })));
                        });
                    }
                    return fileErrors;
                }
            }
        });
    }
}
exports.ElmReviewDiagnostics = ElmReviewDiagnostics;
//# sourceMappingURL=elmReviewDiagnostics.js.map